@@ -1,19 +1,38 @@
 import numpy as np
 import k3d
+from k3d import objects
 from abc import ABCMeta, abstractmethod
 
 
-def create_line_between_joints(start, end, width):
-    line = k3d.line(
-            vertices=[start, end],
-            shader='mesh',
-            width=width)
-    return line
+def get_skeleton_color(skeleton):
+    if skeleton.color == 'default':
+        return 0x0000FF
+    else:
+        return skeleton.color
 
 
-def calculate_size_from_joint_coordinates(joint_coordinates):
-    absolute_max_value = abs(joint_coordinates).max()
-    return absolute_max_value / 10000.0
+def create_skeleton_joint_points(skeleton):
+    return k3d.points(
+            positions=skeleton.joint_coordinates, point_size=skeleton.part_size,
+            shader='mesh', color=get_skeleton_color(skeleton)
+        )
+
+
+def create_skeleton_lines(skeleton, limb_graph):
+    return [line for line in map(
+            lambda line_indices: create_line_between_joints(
+                start=skeleton.joint_coordinates[line_indices[0]],
+                end=skeleton.joint_coordinates[line_indices[1]],
+                width=skeleton.part_size / 4.0, color=get_skeleton_color(skeleton)
+            ),
+            limb_graph)]
+
+
+def create_line_between_joints(start, end, width, color):
+    return k3d.line(
+        vertices=[start, end], shader='mesh',
+        width=width, color=color
+    )
 
 
 class AbstractJointSet(metaclass=ABCMeta):
@@ -25,27 +44,27 @@ class AbstractJointSet(metaclass=ABCMeta):
     def convert_to_common_14(self):
         return None
 
-    def generate_skeleton_from_coordinates(self, joint_coordinates):
-        assert joint_coordinates.shape == (self.number_of_joints, 3)
-        size = calculate_size_from_joint_coordinates(joint_coordinates)
-        joint_points = k3d.points(positions=joint_coordinates, point_size=size, shader='mesh')
-        lines_between_joint_points = [line for line in map(
-            lambda line_indices: create_line_between_joints(
-                start=joint_coordinates[line_indices[0]],
-                end=joint_coordinates[line_indices[1]],
-                width=size / 4.0),
-            self.limb_graph)]
-        return joint_points, lines_between_joint_points
+    def generate_skeleton_plot(self, skeleton):
+        assert skeleton.joint_coordinates.shape == (self.number_of_joints, 3)
+        joint_points = create_skeleton_joint_points(skeleton)
+        lines_between_joint_points = create_skeleton_lines(skeleton, self.limb_graph)
+        skeleton_plot = objects.Group()
+        skeleton_plot += joint_points
+        for line in lines_between_joint_points:
+            skeleton_plot += line
+        return skeleton_plot
 
 
 class MuPoTSJoints(AbstractJointSet):
     def __init__(self):
         super().__init__()
         self.names = np.array([
-            'head_top', 'neck', 'right_shoulder', 'right_elbow',
-            'right_wrist', 'left_shoulder', 'left_elbow', 'left_wrist',
-            'right_hip', 'right_knee', 'right_ankle', 'left_hip',
-            'left_knee', 'left_ankle', 'hip', 'spine', 'head/nose'
+            'HEAD TOP', 'NECK',
+            'RIGHT SHOULDER', 'RIGHT ELBOW', 'RIGHT WRIST',
+            'LEFT SHOULDER', 'LEFT ELBOW', 'LEFT WRIST',
+            'RIGHT HIP', 'RIGHT KNEE', 'RIGHT ANKLE',
+            'LEFT HIP', 'LEFT KNEE', 'LEFT ANKLE',
+            'HIP', 'SPINE', 'NOSE'
         ])
         self.number_of_joints = 17
         self.limb_graph = [
@@ -68,13 +87,15 @@ class OpenPoseJoints(AbstractJointSet):
     def __init__(self):
         super().__init__()
         self.names = np.array([
-            'nose', 'neck', 'right_shoulder', 'right_elbow',
-            'right_wrist', 'left_shoulder', 'left_elbow', 'left_wrist',
-            'hip', 'right_hip', 'right_knee', 'right_ankle',
-            'left_hip', 'left_knee', 'left_ankle', 'right_eye',
-            'left_eye', 'right_ear', 'left_ear', 'left_bigtoe',
-            'left_smalltoe', 'left_heel', 'right_bigtoe',
-            'right_smalltoe', 'right_heel'
+            'NOSE', 'NECK',
+            'RIGHT SHOULDER', 'RIGHT ELBOW', 'RIGHT WRIST',
+            'LEFT SHOULDER', 'LEFT ELBOW', 'LEFT WRIST',
+            'HIP',
+            'RIGHT HIP', 'RIGHT KNEE', 'RIGHT ANKLE',
+            'LEFT HIP', 'LEFT KNEE', 'LEFT ANKLE',
+            'RIGHT EYE', 'LEFT EYE', 'RIGHT EAR', 'LEFT EAR',
+            'LEFT BIG TOE', 'LEFT SMALL TOE', 'LEFT HEEL',
+            'RIGHT BIG TOE', 'RIGHT SMALL TOE', 'RIGHT HEEL'
         ])
         self.limb_graph = [
             (1, 0), (17, 15), (15, 0), (16, 0), (18, 16),               # head
@@ -100,11 +121,16 @@ class CocoExJoints(AbstractJointSet):
     def __init__(self):
         super().__init__()
         self.names = np.array([
-            'nose', 'left_eye', 'right_eye', 'left_ear',
-            'right_ear', 'left_shoulder', 'right_shoulder', 'left_elbow',
-            'right_elbow', 'left_wrist', 'right_wrist', 'left_hip',
-            'right_hip', 'left_knee', 'right_knee', 'left_ankle',
-            'right_ankle', 'hip', 'neck'
+            'NOSE',
+            'LEFT EYE', 'RIGHT EYE',
+            'LEFT EAR', 'RIGHT EAR',
+            'LEFT SHOULDER', 'RIGHT SHOULDER',
+            'LEFT ELBOW', 'RIGHT ELBOW',
+            'LEFT WRIST', 'RIGHT WRIST',
+            'LEFT HIP', 'RIGHT HIP',
+            'LEFT KNEE', 'RIGHT KNEE',
+            'LEFT ANKLE', 'RIGHT ANKLE',
+            'HIP', 'NECK'
         ])
         self.number_of_joints = 19
         self.limb_graph = [
@@ -128,12 +154,13 @@ class PanopticJoints(AbstractJointSet):
     def __init__(self):
         super().__init__()
         self.names = np.array([
-            'neck', 'nose', 'hip',
-            'left_shoulder', 'left_elbow', 'left_wrist',
-            'left_hip', 'left_knee', 'left_ankle',
-            'right_shoulder', 'right_elbow', 'right_wrist',
-            'right_hip', 'right_knee', 'right_ankle',
-            'left_eye', 'left_ear', 'right_eye', 'right_ear'
+            'NECK', 'NOSE','HIP',
+            'LEFT SHOULDER', 'LEFT ELBOW', 'LEFT WRIST',
+            'LEFT HIP', 'LEFT KNEE', 'LEFT ANKLE',
+            'RIGHT SHOULDER', 'RIGHT ELBOW', 'RIGHT WRIST',
+            'RIGHT HIP', 'RIGHT KNEE', 'RIGHT ANKLE',
+            'LEFT EYE', 'LEFT EAR',
+            'RIGHT EYE', 'RIGHT EAR'
         ])
         self.number_of_joints = 19
         self.limb_graph = [
@@ -155,10 +182,12 @@ class PanopticJoints(AbstractJointSet):
 
 class Common14Joints(AbstractJointSet):
     def __init__(self, names=np.array([
-        'hip', 'right_hip', 'right_knee',
-        'right_ankle', 'left_hip', 'left_knee',
-        'left_ankle', 'neck', 'left_shoulder', 'left_elbow',
-        'left_wrist', 'right_shoulder', 'right_elbow', 'right_wrist'
+        'HIP',
+        'RIGHT HIP', 'RIGHT KNEE', 'RIGHT ANKLE',
+        'LEFT HIP', 'LEFT KNEE', 'LEFT ANKLE',
+        'NECK',
+        'LEFT SHOULDER', 'LEFT ELBOW', 'LEFT WRIST',
+        'RIGHT SHOULDER', 'RIGHT ELBOW', 'RIGHT WRIST'
     ])):
         super().__init__()
         self.names = names
